import { type AuthBindings, Refine } from '@refinedev/core'
import { RefineKbar, RefineKbarProvider } from '@refinedev/kbar'
import {
  RefineSnackbarProvider,
  ThemedLayoutV2,
  ThemedTitleV2,
  notificationProvider
} from '@refinedev/mui'
import routerProvider, {
  UnsavedChangesNotifier
} from '@refinedev/nextjs-router'
import type { NextPage } from 'next'
import { SessionProvider, signIn, signOut, useSession } from 'next-auth/react'
import { type AppProps } from 'next/app'
import { useRouter } from 'next/router'
import React from 'react'

import { Header } from '@components/header'
import { ColorModeContextProvider } from '@contexts'
import CssBaseline from '@mui/material/CssBaseline'
import GlobalStyles from '@mui/material/GlobalStyles'
import dataProvider from '../src/providers/dataProvider'
import { appWithTranslation, useTranslation } from 'next-i18next'
import logo from '../src/components/fdo_logo'
import { API_URL } from '../src/constants'

export type NextPageWithLayout<P = {}, IP = P> = NextPage<P, IP> & {
  noLayout?: boolean
}

type AppPropsWithLayout = AppProps & {
  Component: NextPageWithLayout
}

const App = (props: React.PropsWithChildren) => {
  const { t, i18n } = useTranslation()

  const { data, status } = useSession()
  const router = useRouter()
  const { to } = router.query

  const i18nProvider = {
    translate: (key: string, params: object) => t(key, params),
    changeLocale: async (lang: string) => await i18n.changeLanguage(lang),
    getLocale: () => i18n.language
  }

  if (status === 'loading') {
    return <span>loading...</span>
  }

  const authProvider: AuthBindings = {
    login: async () => {
      signIn('keycloak', {
        callbackUrl: to ? to.toString() : '/info',
        redirect: true
      })

      return {
        success: true
      }
    },
    logout: async () => {
      signOut({
        redirect: true,
        callbackUrl: '/login'
      })

      return {
        success: true
      }
    },
    onError: async (error) => {
      console.error(error)
      return {
        error
      }
    },
    check: async () => {
      if (status === 'unauthenticated') {
        return {
          authenticated: false,
          redirectTo: '/login'
        }
      }

      return {
        authenticated: true
      }
    },
    getPermissions: async () => {
      return null
    },
    getIdentity: async () => {
      if (data?.user) {
        const { user } = data
        return {
          name: user.name,
          avatar: user.image
        }
      }

      return null
    }
  }

  return <>
    <RefineKbarProvider>
      <ColorModeContextProvider>
        <CssBaseline />
        <GlobalStyles styles={{ html: { WebkitFontSmoothing: 'auto' } }} />
        <RefineSnackbarProvider>
          <Refine
            routerProvider={routerProvider}
            dataProvider={dataProvider(API_URL)}
            notificationProvider={notificationProvider}
            authProvider={authProvider}
            i18nProvider={i18nProvider}
            options={{
              disableTelemetry: true,
              syncWithLocation: true,
              warnWhenUnsavedChanges: true,
              useNewQueryKeys: true
            }}
            resources={[
              {
                name: 'fdo',
                list: '/fdo',
                show: '/fdo/show/:id',
                create: '/fdo'
              }, {
                name: 'repositories',
                list: '/repositories',
                show: '/repositories/show/:id'
              }, {
                name: 'info',
                list: '/info'
              }]}
          >
            {props.children}
            <RefineKbar />
            <UnsavedChangesNotifier />
          </Refine>
        </RefineSnackbarProvider>
      </ColorModeContextProvider>
    </RefineKbarProvider>
  </>
}

function MyApp ({
  Component,
  pageProps: { session, ...pageProps }
}: AppPropsWithLayout): JSX.Element {
  const renderComponent = () => {
    if (Component.noLayout) {
      return <Component {...pageProps} />
    }

    return (
      <ThemedLayoutV2
        Header={() => <Header sticky />}
        Title={({ collapsed }) => (
          <ThemedTitleV2
            collapsed={collapsed}
            text="FDO Manager"
            icon={logo}
          />
        )}
        >
        <Component {...pageProps} />
      </ThemedLayoutV2>
    )
  }

  return (
    <SessionProvider session={session}>
      <App>{renderComponent()}</App>
    </SessionProvider>
  )
}

export default appWithTranslation(MyApp)
